@@ -49,7 +49,7 @@
 #include <private/qguiapplication_p.h>
 #include <private/qhighdpiscaling_p.h>
 #include <QtGui/qevent.h>
-#include <QtEventDispatcherSupport/private/qwindowsguieventdispatcher_p.h>
+#include <QtGui/private/qwindowsguieventdispatcher_p.h>
 #include <QtCore/private/qdebug_p.h>
 
 #if defined(WM_APPCOMMAND)
@@ -88,7 +88,6 @@ QT_BEGIN_NAMESPACE
     \class QWindowsKeyMapper
     \brief Translates Windows keys to QWindowSystemInterface events.
     \internal
-    \ingroup qt-lighthouse-win
 
     In addition, handles some special keys to display system menus, etc.
     The code originates from \c qkeymapper_win.cpp.
@@ -97,10 +96,10 @@ QT_BEGIN_NAMESPACE
 static void clearKeyRecorderOnApplicationInActive(Qt::ApplicationState state);
 
 QWindowsKeyMapper::QWindowsKeyMapper()
-    : m_useRTLExtensions(false), m_keyGrabber(0)
+    : m_useRTLExtensions(false), m_keyGrabber(nullptr)
 {
     memset(keyLayout, 0, sizeof(keyLayout));
-    QGuiApplication *app = static_cast<QGuiApplication *>(QGuiApplication::instance());
+    auto *app = static_cast<QGuiApplication *>(QGuiApplication::instance());
     QObject::connect(app, &QGuiApplication::applicationStateChanged,
                      app, clearKeyRecorderOnApplicationInActive);
     changeKeyboard();
@@ -162,7 +161,7 @@ static void clearKeyRecorderOnApplicationInActive(Qt::ApplicationState state)
 
 KeyRecord *KeyRecorder::findKey(int code, bool remove)
 {
-    KeyRecord *result = 0;
+    KeyRecord *result = nullptr;
     for (int i = 0; i < nrecs; ++i) {
         if (records[i].code == code) {
             if (remove) {
@@ -541,7 +540,7 @@ static const Qt::KeyboardModifiers ModsTbl[] = {
     Qt::NoModifier,                                             // Fall-back to raw Key_*
 };
 static const size_t NumMods = sizeof ModsTbl / sizeof *ModsTbl;
-Q_STATIC_ASSERT((NumMods == KeyboardLayoutItem::NumQtKeys));
+static_assert((NumMods == KeyboardLayoutItem::NumQtKeys));
 
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug d, const KeyboardLayoutItem &k)
@@ -554,7 +553,7 @@ QDebug operator<<(QDebug d, const KeyboardLayoutItem &k)
             if (const quint32 qtKey = k.qtKey[i]) {
                 d << '[' << i << ' ';
                 QtDebugUtils::formatQFlags(d, ModsTbl[i]);
-                d << ' ' << hex << showbase << qtKey << dec << noshowbase << ' ';
+                d << ' ' << Qt::hex << Qt::showbase << qtKey << Qt::dec << Qt::noshowbase << ' ';
                 QtDebugUtils::formatQEnum(d, Qt::Key(qtKey));
                 if (qtKey >= 32 && qtKey < 128)
                     d << " '" << char(qtKey)  << '\'';
@@ -605,7 +604,7 @@ inline quint32 winceKeyBend(quint32 keyCode)
 }
 
 // Translate a VK into a Qt key code, or unicode character
-static inline quint32 toKeyOrUnicode(quint32 vk, quint32 scancode, unsigned char *kbdBuffer, bool *isDeadkey = 0)
+static inline quint32 toKeyOrUnicode(quint32 vk, quint32 scancode, unsigned char *kbdBuffer, bool *isDeadkey = nullptr)
 {
     Q_ASSERT(vk > 0 && vk < 256);
     quint32 code = 0;
@@ -776,7 +775,7 @@ void QWindowsKeyMapper::updatePossibleKeyCodes(unsigned char *kbdBuffer, quint32
         ::ToAscii(vk_key, scancode, kbdBuffer, reinterpret_cast<LPWORD>(&buffer), 0);
     }
     qCDebug(lcQpaEvents) << __FUNCTION__ << "for virtual key="
-        << hex << showbase << vk_key << dec << noshowbase << keyLayout[vk_key];
+        << Qt::hex << Qt::showbase << vk_key << Qt::dec << Qt::noshowbase << keyLayout[vk_key];
 }
 
 static inline QString messageKeyText(const MSG &msg)
@@ -822,7 +821,7 @@ static void showSystemMenu(QWindow* w)
                                TPM_LEFTALIGN  | TPM_TOPALIGN | TPM_NONOTIFY | TPM_RETURNCMD,
                                pos.x(), pos.y(),
                                topLevelHwnd,
-                               0);
+                               nullptr);
     if (ret)
         qWindowsWndProc(topLevelHwnd, WM_SYSCOMMAND, WPARAM(ret), 0);
 }
@@ -879,21 +878,16 @@ bool QWindowsKeyMapper::translateMultimediaKeyEventInternal(QWindow *window, con
 #if defined(WM_APPCOMMAND)
     const int cmd = GET_APPCOMMAND_LPARAM(msg.lParam);
     // QTBUG-57198, do not send mouse-synthesized commands as key events in addition
+    bool skipPressRelease = false;
     switch (GET_DEVICE_LPARAM(msg.lParam)) {
     case FAPPCOMMAND_MOUSE:
         return false;
     case FAPPCOMMAND_KEY:
-        // QTBUG-62838, swallow WM_KEYDOWN, WM_KEYUP for commands that are
-        // reflected in VK(s) like VK_MEDIA_NEXT_TRACK. Don't do that for
-        // APPCOMMAND_BROWSER_HOME as that one does not trigger two events
-        if (cmd != APPCOMMAND_BROWSER_HOME) {
-            MSG peekedMsg;
-            if (PeekMessage(&peekedMsg, msg.hwnd, 0, 0, PM_NOREMOVE)
-                && peekedMsg.message == WM_KEYDOWN) {
-                PeekMessage(&peekedMsg, msg.hwnd, 0, 0, PM_REMOVE);
-                PeekMessage(&peekedMsg, msg.hwnd, 0, 0, PM_REMOVE);
-            }
-        }
+        // QTBUG-62838, use WM_KEYDOWN/WM_KEYUP for commands that are reflected
+        // in VK(s) like VK_MEDIA_NEXT_TRACK, to get correct codes and autorepeat.
+        // Don't do that for APPCOMMAND_BROWSER_HOME as that one does not trigger two events.
+        if (cmd != APPCOMMAND_BROWSER_HOME)
+            skipPressRelease = true;
         break;
     }
 
@@ -908,11 +902,12 @@ bool QWindowsKeyMapper::translateMultimediaKeyEventInternal(QWindow *window, con
         return false;
 
     const int qtKey = int(CmdTbl[cmd]);
-    sendExtendedPressRelease(receiver, qtKey, Qt::KeyboardModifier(state), 0, 0, 0);
+    if (!skipPressRelease)
+        sendExtendedPressRelease(receiver, qtKey, Qt::KeyboardModifier(state), 0, 0, 0);
     // QTBUG-43343: Make sure to return false if Qt does not handle the key, otherwise,
     // the keys are not passed to the active media player.
 # if QT_CONFIG(shortcut)
-    const QKeySequence sequence(Qt::Modifier(state) + qtKey);
+    const QKeySequence sequence(Qt::Modifier(state) | Qt::Key(qtKey));
     return QGuiApplicationPrivate::instance()->shortcutMap.hasShortcutForKeySequence(sequence);
 # else
     return false;
@@ -954,7 +949,7 @@ bool QWindowsKeyMapper::translateKeyEventInternal(QWindow *window, MSG msg,
     const UINT msgType = msg.message;
 
     const quint32 scancode = (msg.lParam >> 16) & scancodeBitmask;
-    quint32 vk_key = quint32(msg.wParam);
+    auto vk_key = quint32(msg.wParam);
     quint32 nModifiers = 0;
 
     QWindow *receiver = m_keyGrabber ? m_keyGrabber : window;
@@ -1023,13 +1018,15 @@ bool QWindowsKeyMapper::translateKeyEventInternal(QWindow *window, MSG msg,
                 if (dirStatus == VK_LSHIFT
                         && ((msg.wParam == VK_SHIFT && GetKeyState(VK_LCONTROL))
                             || (msg.wParam == VK_CONTROL && GetKeyState(VK_LSHIFT)))) {
-                    sendExtendedPressRelease(receiver, Qt::Key_Direction_L, 0, scancode, vk_key, nModifiers, QString(), false);
+                    sendExtendedPressRelease(receiver, Qt::Key_Direction_L, {},
+                                             scancode, vk_key, nModifiers, QString(), false);
                     result = true;
                     dirStatus = 0;
                 } else if (dirStatus == VK_RSHIFT
                            && ( (msg.wParam == VK_SHIFT && GetKeyState(VK_RCONTROL))
                                 || (msg.wParam == VK_CONTROL && GetKeyState(VK_RSHIFT)))) {
-                    sendExtendedPressRelease(receiver, Qt::Key_Direction_R, 0, scancode, vk_key, nModifiers, QString(), false);
+                    sendExtendedPressRelease(receiver, Qt::Key_Direction_R, {},
+                                             scancode, vk_key, nModifiers, QString(), false);
                     result = true;
                     dirStatus = 0;
                 } else {
@@ -1140,7 +1137,7 @@ bool QWindowsKeyMapper::translateKeyEventInternal(QWindow *window, MSG msg,
         // This will stop the auto-repeat of the key, should a modifier change, for example
         if (rec && rec->state != state) {
             key_recorder.findKey(int(msg.wParam), true);
-            rec = 0;
+            rec = nullptr;
         }
 
         // Find unicode character from Windows Message Queue
@@ -1150,7 +1147,7 @@ bool QWindowsKeyMapper::translateKeyEventInternal(QWindow *window, MSG msg,
                          : msgType == WM_IME_KEYDOWN ? WM_IME_CHAR : WM_SYSCHAR);
 
         QChar uch;
-        if (PeekMessage(&wm_char, 0, charType, charType, PM_REMOVE)) {
+        if (PeekMessage(&wm_char, nullptr, charType, charType, PM_REMOVE)) {
             if (QWindowsContext::filterNativeEvent(&wm_char, lResult))
                 return true;
             if (receiver && QWindowsContext::filterNativeEvent(receiver, &wm_char, lResult))
@@ -1184,7 +1181,7 @@ bool QWindowsKeyMapper::translateKeyEventInternal(QWindow *window, MSG msg,
         // results, if we map this virtual key-code directly (for eg '?' US layouts). So try
         // to find the correct key using the current message parameters & keyboard state.
         if (uch.isNull() && msgType == WM_IME_KEYDOWN) {
-            const QWindowsInputContext *windowsInputContext =
+            const auto *windowsInputContext =
                 qobject_cast<const QWindowsInputContext *>(QWindowsIntegration::instance()->inputContext());
             if (!(windowsInputContext && windowsInputContext->isComposing()))
                 vk_key = ImmGetVirtualKey(reinterpret_cast<HWND>(window->winId()));
@@ -1258,7 +1255,7 @@ bool QWindowsKeyMapper::translateKeyEventInternal(QWindow *window, MSG msg,
 #ifndef QT_NO_SHORTCUT
             // Is Qt interested in the context menu key?
             if (modifiers == Qt::SHIFT && code == Qt::Key_F10
-                && !QGuiApplicationPrivate::instance()->shortcutMap.hasShortcutForKeySequence(QKeySequence(Qt::SHIFT + Qt::Key_F10))) {
+                && !QGuiApplicationPrivate::instance()->shortcutMap.hasShortcutForKeySequence(QKeySequence(Qt::SHIFT | Qt::Key_F10))) {
                 return false;
             }
 #endif // !QT_NO_SHORTCUT
@@ -1303,7 +1300,23 @@ bool QWindowsKeyMapper::translateKeyEventInternal(QWindow *window, MSG msg,
                       || code == Qt::Key_Control
                       || code == Qt::Key_Meta
                       || code == Qt::Key_Alt)) {
-            // Someone ate the key down event
+
+            // Workaround for QTBUG-77153:
+            // The Surface Pen eraser button generates Meta+F18/19/20 keystrokes,
+            // but when it is not touching the screen the Fn Down is eaten and only
+            // a Fn Up with the previous state as "not pressed" is generated, which
+            // would be ignored. We detect this case and synthesize the expected events.
+            if ((msg.lParam & 0x40000000) == 0 &&
+                    Qt::KeyboardModifier(state) == Qt::NoModifier &&
+                    ((code == Qt::Key_F18) || (code == Qt::Key_F19) || (code == Qt::Key_F20))) {
+                QWindowSystemInterface::handleExtendedKeyEvent(receiver, QEvent::KeyPress, code,
+                                                               Qt::MetaModifier, scancode,
+                                                               quint32(msg.wParam), MetaLeft);
+                QWindowSystemInterface::handleExtendedKeyEvent(receiver, QEvent::KeyRelease, code,
+                                                               Qt::NoModifier, scancode,
+                                                               quint32(msg.wParam), 0);
+                result = true;
+            }
         } else {
             if (!code)
                 code = asciiToKeycode(rec->ascii ? char(rec->ascii) : char(msg.wParam), state);
@@ -1363,18 +1376,18 @@ QList<int> QWindowsKeyMapper::possibleKeys(const QKeyEvent *e) const
     quint32 baseKey = kbItem.qtKey[0];
     Qt::KeyboardModifiers keyMods = e->modifiers();
     if (baseKey == Qt::Key_Return && (e->nativeModifiers() & ExtendedKey)) {
-        result << int(Qt::Key_Enter + keyMods);
+        result << (Qt::Key_Enter | keyMods).toCombined();
         return result;
     }
-    result << int(baseKey + keyMods); // The base key is _always_ valid, of course
+    result << int(baseKey) + int(keyMods); // The base key is _always_ valid, of course
 
     for (size_t i = 1; i < NumMods; ++i) {
         Qt::KeyboardModifiers neededMods = ModsTbl[i];
         quint32 key = kbItem.qtKey[i];
         if (key && key != baseKey && ((keyMods & neededMods) == neededMods)) {
             const Qt::KeyboardModifiers missingMods = keyMods & ~neededMods;
-            const int matchedKey = int(key) + missingMods;
-            const QList<int>::iterator it =
+            const int matchedKey = int(key) + int(missingMods);
+            const auto it =
                 std::find_if(result.begin(), result.end(),
                              [key] (int k) { return (k & ~Qt::KeyboardModifierMask) == key; });
             // QTBUG-67200: Use the match with the least modifiers (prefer
@@ -1386,7 +1399,7 @@ QList<int> QWindowsKeyMapper::possibleKeys(const QKeyEvent *e) const
         }
     }
     qCDebug(lcQpaEvents) << __FUNCTION__  << e << "nativeVirtualKey="
-        << showbase << hex << e->nativeVirtualKey() << dec << noshowbase
+        << Qt::showbase << Qt::hex << e->nativeVirtualKey() << Qt::dec << Qt::noshowbase
         << e->modifiers() << kbItem << "\n  returns" << formatKeys(result);
     return result;
 }
